function pdec=deconvdensityexplicit(y,T,Gn,options,wavft,lambda,L,thdtype)
% Term-by-term density deconvolution.
%
% 	Input
%		y	Input realizations.
%		T	Output density length (discretized, default, 2*length(y)).
%		Gn	Characteristic function of noise (must be of length T).
%		options	Options of the characteristic function and pdf of x
%			sigtau	sig_v appearing in the ordinary smoothness assumption.
%			delta   Exponent in the ordinary smoothness assumption.
%			C	sup_x pdf_x(x) <= C.
%		wavft	Fourier transform of wavelets.
%		lambda	Threshold parameter.
%		L	Block size (default, 1: term-by-term).
%
%	Output
%		pdec	deconvolved density.
%

if (exist('y')~=1), 				error('Provide input realizations');		end;
if (exist('T')~=1), 				T = length(y)*2;				end;
if (exist('Gn')~=1), 				error('Provide input characteristic function');	end;
if (exist('options')~=1),			error('Provide options');			end;
if (exist('wavft')~=1),				error('Provide wavelets');			end;
if (exist('lambda')~=1 | isempty(lambda)),	lambda = 4.5;					end;
if (exist('thdtype')~=1 | isempty(thdtype)),	thdtype= 1;					end;
if isfield(options,'sigtau'),
	sigtau = options.sigtau;
else
	error('Provide sigtau in options: see the ordinary smoothness assumption');
end
if isfield(options,'delta'),
	delta = options.delta;
else
	error('Provide delta in options: see the ordinary smoothness assumption');
end

if isfield(options,'C'),
	C = options.C;
else
	error('Provide C such that sup_x pdf_x(x) <= C');
end

n = length(y);
if n==1,	disp('Insufficient number of realizations.'); end

if isscalar(sigtau),	sigtau = options.sigtau*ones(n,1); end

J	= nextpow2(T);
if (exist('L')~=1 | isempty(L)), 	L = 2^round(log2(log(T)));				end; % Size of block.

weight= (1+sigtau.^2).^(-delta);
omegan= sum(weight);
scale = max(log2(L),3);   % Coarsest scale L%2^Jc matches the size of the block.
J1    = floor(log2(omegan)/(2*delta+1));

c     = 1;
nu    = [-T/2:T/2-1]'/T;
theta = sqrt(sum([0.5*T*fftshift(abs(wavft(:,end))).^2.*(1+(2*pi*nu).^2).^(-delta)])*C/c^2);

Mft= exp(-i*2*pi*nu*y')./Gn;
%wc = real((wavft'*fftshift(Mft*weight))/(omegan*sqrt(T))); %wcold=wc;PlotWaveCoeff(wcold,scale,0);drawnow;pause
Mwft=fftshift(Mft,1).*repmat(weight',T,1);
wc = real(wavft'*Mwft)/sqrt(T);
%PlotWaveCoeff(sum(wc,2)/omegan,scale,0);drawnow;pause

sigmaj = compute_thdMW(1,scale,J,3,fftshift(Gn));
sigmaj = fliplr(sqrt(2.^(-[1:J-scale]*delta))/4);
thdj(1:2^scale) = realmax;theta*4*sigmaj*sqrt(omegan/log(omegan));
for j = 0:J-1-scale
  thdj(2^(J-j-1)+1 : 2^(J-j)) = theta*4*sigmaj(end-j)*sqrt(omegan/log(omegan));
end
%plot(thdj);drawnow;pause
wc = wc .* (abs(wc) <= repmat(thdj',1,n));
wc = sum(wc,2)/omegan;
%PlotWaveCoeff(wc,scale,0);drawnow;pause

for j = 0:J-1-scale
 HH = wc(2^(J-j-1)+1 : 2^(J-j));
 % reshape as block
 H = reshape(HH,L,floor(2^(-j-1)*T/L));
 % Block hard threshold
 Hmask = (lambda*theta)^2*sigmaj(end-j)^2*log(omegan)/omegan <= mean(abs(H).^2,1);
 HH = reshape(repmat(Hmask,L,1) .* H,2^(-j-1)*T,1);
 wc(2^(J-j-1)+1 : 2^(J-j)) = HH;
end
%plot(wc);drawnow;pause
%PlotWaveCoeff(wcold,scale,0);subplot(212);PlotWaveCoeff(wc,scale,0);drawnow;pause
%wc(T/2^J1+1:end)  = 0; % May be useful to kill finest detail coefficients.
pdec = real(fftshift(ifft(wavft*wc)*sqrt(T)));
		
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sigmaj = compute_thdMW(thdtype,jc,J,deg,G)

wavfun = scaling_YM(jc,J,deg)*2^(-J/2);

if thdtype == 1
	sigmaj(1) = norm(wavfun(wavfun~=0)./G(wavfun~=0));

	for j=jc:J-2
	  wavfun = wavelet_YM(j,J,deg)*2^(-J/2);
	  sigmaj(end+1) = norm(wavfun(wavfun~=0)./G(wavfun~=0));
	end
	wavfun = fine_YM(J-1,J,deg)*2^(-J/2);
	sigmaj(end+1) = norm(wavfun(wavfun~=0)./G(wavfun~=0));
	
elseif thdtype == 2
	sigmaj(1) = 2^((-J-jc)/2)*norm(1./G(wavfun~=0)); % Approximate phi_jc by a Shannon scaling function constant with 2^(-jc/2) in its bandwidth. 
	for j=jc:J-2
	  wavfun = wavelet_YM(j,J,deg);
	  sigmaj(end+1) = 2^((-J-j-1)/2)*norm(1./G(wavfun~=0)); % Approximate psi_j by a Shannon wavelet constant with 2^(-j/2) in its bandwidth.
	end
	wavfun = fine_YM(J-1,J,deg);
	sigmaj(end+1) = 2^(-(2*J)/2)*norm(1./G(wavfun~=0)); % Approximate psi_J by a Shannon wavelet constant with 2^(-(J-1)/2) in its bandwidth.	
elseif thdtype == 3
	wc = FWT_YM(real(ifft(fft(randn(2^J,1))./G(:))),jc,deg);
	for j = 1:J-jc
		sigmaj(j) = std(wc(2^(J-j)+1 : 2^(J-j+1)),1);
	end
	sigmaj = [std(wc(1:2^jc)) fliplr(sigmaj)];
else
	disp('Unknown type.');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function phi = phiHAT(x,deg)
% auxillary function for computing Meyer scaling function
% in the Fourier domain

winh = WindowMeyer((3*abs(x)-1),deg);

cos_part = cos(pi*winh/2);

phi = zeros(1,length(x));
phi = cos_part.*(abs(x)>1/3 & abs(x)<=2/3);
phi(abs(x)<=1/3) = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = psiHAT(x,deg)
% auxillary function for computing Meyer wavelet function
% in the Fourier domain

wing = WindowMeyer((3*abs(x)-1),deg);
winh = WindowMeyer((3*abs(x)/2-1),deg);

gpart = (exp(-i*pi*x)).*sin(pi*wing/2);
hpart = (exp(-i*pi*x)).*cos(pi*winh/2);

y1 = gpart.*(abs(x)>1/3 & abs(x)<=2/3);
y2 = hpart.*(abs(x)>2/3 & abs(x)<=4/3);

psij = y1+y2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%3
function phij = scaling_YM(j,j_max,deg)
% Generate phiHAT(j,0)--the fft of the scaling function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;
aux1=phiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

phij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(-j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = wavelet_YM(j,j_max,deg)
% generate psiHAT(j,0)--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).

omega_pos = [0:1/(2^j):10/(2*pi)];
n_pos = length(omega_pos);

nn = 2^j_max;

omega_neg = [-10/(2*pi):1/(2^j):-1/2^j];
n_omega = 2*length(omega_pos)-1;

n = 2^j_max-n_omega;

aux1 = psiHAT(omega_pos,deg);

%Take zero out
psiHAT_LEFT=zeros(1,nn);
psiHAT_LEFT(1:n_pos) = aux1;
aux2 = [psiHAT_LEFT(2:end) 0];

psiHAT_RIGHT = conj(fliplr(aux2));

psij = (psiHAT_RIGHT+psiHAT_LEFT)*2^(-j/2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function psij = fine_YM(j,j_max,deg)
% generate psiHAT--the fft of the wavelet function--in the periodic setting.
% deg=deg of the Meyer wavelet (1,2,3).
% at the finest resolution level

omega = linspace(-pi,pi,2^(j_max));
phij = scaling_YM(j,j_max,deg);
psij = exp(-i*linspace(-pi,pi,2^(j_max))/2^j).*conj(fftshift(phij));

% omega_pos = [0:1/(2^j):10/(2*pi)];
% n_pos = 2^j-2^(j-2);
% 
% nn = 2^j_max;
% 
% aux1 = psiHAT(omega_pos,deg);
% 
% %Take zero out
% n2 = n_pos+2^(j-2)+1;
% psiHAT_LEFT=zeros(1,n2);
% 
% psiHAT_LEFT(1:n_pos) = aux1(1:n_pos);
% psiHAT_LEFT((n_pos+1):(n_pos+2^(j-2)+1)) = -1;
% aux2 = psiHAT_LEFT(2:end);
% 
% psiHAT_RIGHT = conj(fliplr(psiHAT_LEFT));
% psiHAT_RIGHT = psiHAT_RIGHT(2:end-1);
% 
% psij = [psiHAT_LEFT psiHAT_RIGHT]*2^(-j/2)*nn;




%
% Part of DensityDeconvToolbox Version:100
% Written by: Jalal Fadili, GREYC CNRS ENSICAEN
%             Christophe Chesneau, LMNO University of Caen
% Created July 2011
% This material is distributed under CeCiLL licence.
% E-mail: Jalal.Fadili@greyc.ensicaen.fr
% 


